import 'package:dio/dio.dart';
import 'package:flutter/material.dart';
import 'package:lori_sdk/data/lori_api.dart';
import 'package:lori_sdk/data/models/ranking/ranking.dart';

class FirebaseApi {
  static const API_SELLER = 'firebase/seller';
  static const API_RANKING = 'firebase/ranking';
  static const API_SELLER_AMOUNT = 'firebase/seller/%';

  Dio _requester;

  FirebaseApi({@required Dio requester}) : this._requester = requester;

  Future<ApiCallback<Ranking>> getUserRankingInfo(String document) async {
    Stopwatch stopwatch = new Stopwatch()..start();
    try {
      var queryParameters = {
        'document': document
      };
      var response = await _requester.get(API_SELLER,
          queryParameters: queryParameters,
          options: Options(responseType: ResponseType.json));
      var ranking = Ranking.fromJson(response.data);

      print("Ranking information loaded in ${stopwatch.elapsedMilliseconds}ms");
      return ApiCallback<Ranking>(isSuccessful: true, resultData: ranking);
    } on DioError catch (e) {
      print("Ranking information loaded in ${stopwatch.elapsedMilliseconds}ms");
      return ApiCallback<Ranking>().defaultErrorCallback(e);
    }
  }

  Future<ApiCallback<List<Ranking>>> getRanking() async {
    Stopwatch stopwatch = new Stopwatch()..start();
    try {
      var response = await _requester.get(API_RANKING,
          options: Options(responseType: ResponseType.json));
      var responseData = response.data as List;
      var rankingItems = List<Ranking>();
      for (var row in responseData) {
        rankingItems.add(Ranking.fromJson(row));
      }
      print("Ranking loaded in ${stopwatch.elapsedMilliseconds}ms");
      return ApiCallback<List<Ranking>>(isSuccessful: true, resultData: rankingItems);
    } on DioError catch (e) {
      print("Ranking loaded in ${stopwatch.elapsedMilliseconds}ms");
      return ApiCallback<List<Ranking>>().defaultErrorCallback(e);
    }
  }

  Future<ApiCallback<void>> putAmount(String dealerCode, double amount) async {
    Stopwatch stopwatch = new Stopwatch()..start();
    try {
      var queryParameters = {
        'amount': amount
      };
      await _requester.put(
        API_SELLER_AMOUNT.replaceAll('%', dealerCode),
        queryParameters: queryParameters,
        options: Options(responseType: ResponseType.json)
      );
      return ApiCallback<void>(isSuccessful: true);
    } on DioError catch (e) {
      print("Amount put in ${stopwatch.elapsedMilliseconds}ms");
      return ApiCallback<void>().defaultErrorCallback(e);
    }
  }
}