import 'dart:async';
import 'dart:convert';

import 'package:lori_sdk/data/database/interfaces/cartDao.dart';
import 'package:lori_sdk/data/models/cart/cart.dart';
import 'package:lori_sdk/data/models/singleton/cartSingleton.dart';
import 'package:shared_preferences/shared_preferences.dart';

const int kVersion1 = 1;

class WebCartDao implements CartDao {
  static const _CART_KEY = 'x_cart';

  SharedPreferences db;

  Future open() async {
    if (db == null) {
      db = await SharedPreferences.getInstance();
    }
  }

  Future<Cart> get() async {
    await open();
    final cartInJson = db.getString(_CART_KEY);
    return cartInJson != null ? Cart.fromJson(jsonDecode(cartInJson)) : null;
  }

  Future<void> insertOrUpdate(Cart cart) async {
    var storedResponse = await get();
    if (storedResponse == null) {
      return insert(cart);
    } else {
      return update(cart);
    }
  }

  Future<void> insert(Cart cart) async {
    await open();
    await db.setString(_CART_KEY, jsonEncode(cart.toJson()));
    CartSingleton().cartChanged(cart);
    return;
  }

  Future<void> update(Cart cart) async {
    await open();
    db.setString(_CART_KEY, jsonEncode(cart.toJson()));
    CartSingleton().cartChanged(cart);
    return;
  }

  Future<void> deleteAll() async {
    await open();
    await db.setString(_CART_KEY, null);
    return;
  }
}

CartDao getCartDao() => WebCartDao();
