import 'package:flutter/material.dart';
import 'package:json_annotation/json_annotation.dart';

part 'brandConfigurations.g.dart';

@JsonSerializable()
class BrandConfigurations {

  BrandConfigurations();

  @JsonKey(name: 'logoHorizontalPrimaryImageUrl')
  String horizontalLogoLightBackgroundImageUrl;

  @JsonKey(name: 'logoHorizontalSecondaryImageUrl')
  String horizontalLogoDarkBackgroundImageUrl;

  @JsonKey(name: 'brandIconUrl')
  String appIconImageUrl;

  // Colors
  @JsonKey(name: 'primaryColor')
  String primaryColorText;
  @JsonKey(ignore: true)
  Color primaryColor;


  @JsonKey(name: 'secondaryColor')
  String secondaryColorText;
  @JsonKey(ignore: true)
  Color secondaryColor;

  @JsonKey(name: 'appBarColor')
  String appBarColorText;
  @JsonKey(ignore: true)
  Color appBarColor;


  @JsonKey(name: 'colorHome')
  String splashColorText;
  @JsonKey(ignore: true)
  Color splashColor;

  @JsonKey(name: 'imageHome')
  String splashImageUrl;
  @JsonKey(name: 'isFullImageHome')
  bool isSplashImageFull;

  @JsonKey(name: 'priceListColor', defaultValue: '#d96401')
  String priceListColorText;
  @JsonKey(ignore: true)
  Color priceListColor;

  @JsonKey(name: 'priceDetailColor', defaultValue: '#5c9e0e')
  String priceDetailColorText;
  @JsonKey(ignore: true)
  Color priceDetailColor;

  /// Calculate if we should use black or white on the app bar
  /// If appBarColor is null this means the appBarColor is the primaryColor
  Color get appBarTextColor {
    Color appBarColor = this.appBarColor;
    if (appBarColor == null) {
      return Colors.white;
    } else {
      double luma = ((0.299 * appBarColor.red) + (0.587 * appBarColor.green) + (0.114 * appBarColor.blue)) / 255;
      // Return black for bright colors, white for dark colors
      return luma > 0.5 ? Colors.black : Colors.white;
    }
  }

  /// Shorthand for status bar brightness
  Brightness get appBarBrightness {
    if (this.appBarTextColor == Colors.white) {
      return Brightness.dark;
    }
    else {
      return Brightness.light;
    }
  }

  /// A short-hand property to retrieve the tabBarBackgroundColor
  Color get appBarBackgroundColor {
    return this.appBarColor ?? this.primaryColor;
  }

  factory BrandConfigurations.fromJson(Map<String, dynamic> json) {
    BrandConfigurations brandConfigurations = _$BrandConfigurationsFromJson(json);

//Set the colors from the hex string
    if (brandConfigurations.primaryColorText != null && brandConfigurations.primaryColorText.isNotEmpty) {
      brandConfigurations.primaryColor = _colorFromHexString(brandConfigurations.primaryColorText);
    }

    if (brandConfigurations.secondaryColorText != null && brandConfigurations.secondaryColorText.isNotEmpty) {
      brandConfigurations.secondaryColor = _colorFromHexString(brandConfigurations.secondaryColorText);
    }

    if (brandConfigurations.appBarColorText != null && brandConfigurations.appBarColorText.isNotEmpty) {
      brandConfigurations.appBarColor = _colorFromHexString(brandConfigurations.appBarColorText);
    }

    if (brandConfigurations.splashColorText != null && brandConfigurations.splashColorText.isNotEmpty) {
      brandConfigurations.splashColor = _colorFromHexString(brandConfigurations.splashColorText);
    }

    if (brandConfigurations.priceListColorText != null && brandConfigurations.priceListColorText.isNotEmpty) {
      brandConfigurations.priceListColor = _colorFromHexString(brandConfigurations.priceListColorText);
    }

    if (brandConfigurations.priceDetailColorText != null && brandConfigurations.priceDetailColorText.isNotEmpty) {
      brandConfigurations.priceDetailColor = _colorFromHexString(brandConfigurations.priceDetailColorText);
    }

    return brandConfigurations;
  }

  Map<String, dynamic> toJson() => _$BrandConfigurationsToJson(this);

  static Color _colorFromHexString(String hex) {
    hex = hex.toUpperCase().replaceAll("#", "");
    if (hex.length == 6) {
      //Add 255 opacity to the color
      hex = "FF$hex";
    }

    return Color(int.parse(hex, radix: 16));
  }
}