import 'dart:async';

import 'package:lori_sdk/data/database/interfaces/cartDao.dart';
import 'package:lori_sdk/data/models/cart/cart.dart';

class CartSingleton {
  // Singleton part
  static final CartSingleton _singleton = CartSingleton._internal();

  factory CartSingleton() {
    return _singleton;
  }

  CartSingleton._internal();

  // Class part
  // ignore: close_sinks
  StreamController<Cart> _cartStream =
      StreamController<Cart>.broadcast(sync: true);

  Future<StreamSubscription<Cart>> registerToStream(Function callback) async {
    StreamSubscription<Cart> subscription = _cartStream.stream.listen(callback);
    // Make sure to call cart changed this time
    var cart = await CartDao().get();
    cartChanged(cart);
    return subscription;
  }

  void cartChanged(Cart cart) {
    _cartStream.sink.add(cart);
  }

  void cartClose() async {
    await _cartStream.sink.close();
  }
}
